@@ -41,16 +41,26 @@ import org.joda.convert.factory.NumericObjectArrayStringConverterFactory;
  * StringConvert is thread-safe with concurrent caches.
  */
 public final class StringConvert {
+    // NOTE!
+    // There must be no references (direct or indirect) to RenameHandler
+    // This class must be loaded first to avoid horrid loops in class initialization
 
     /**
-     * An immutable global instance.
-     * <p>
-     * This instance cannot be added to using {@link #register}, however annotated classes
-     * are picked up. To register your own converters, simply create an instance of this class.
+     * Errors in class initialization are hard to debug.
+     * Set -Dorg.joda.convert.debug=true on the command line to add extra logging to System.err
      */
-    public static final StringConvert INSTANCE = new StringConvert();
+    static final boolean LOG;
+    static {
+        String log = null;
+        try {
+            log = System.getProperty("org.joda.convert.debug");
+        } catch (SecurityException ex) {
+            // ignore
+        }
+        LOG = "true".equalsIgnoreCase(log);
+    }
     /**
-     * The cached null object.
+     * The cached null object. Ensure this is above public constants.
      */
     private static final TypedStringConverter<?> CACHED_NULL = new TypedStringConverter<Object>() {
         @Override
@@ -66,6 +76,13 @@ public final class StringConvert {
             return null;
         }
     };
+    /**
+     * An immutable global instance.
+     * <p>
+     * This instance cannot be added to using {@link #register}, however annotated classes
+     * are picked up. To register your own converters, simply create an instance of this class.
+     */
+    public static final StringConvert INSTANCE = new StringConvert();
 
     /**
      * The list of factories.
@@ -156,11 +173,131 @@ public final class StringConvert {
             registered.put(Float.TYPE, JDKStringConverter.FLOAT);
             registered.put(Double.TYPE, JDKStringConverter.DOUBLE);
             registered.put(Character.TYPE, JDKStringConverter.CHARACTER);
-            // Guava and Java 8
             tryRegisterGuava();
             tryRegisterJava8Optionals();
             tryRegisterTimeZone();
-            // JDK 1.8 classes
+            tryRegisterJava8();
+            tryRegisterThreeTenBackport();
+            tryRegisterThreeTenOld();
+        }
+        if (factories.length > 0) {
+            this.factories.addAll(Arrays.asList(factories));
+        }
+        this.factories.add(AnnotationStringConverterFactory.INSTANCE);
+        if (includeJdkConverters) {
+            this.factories.add(EnumStringConverterFactory.INSTANCE);
+            this.factories.add(TypeStringConverterFactory.INSTANCE);
+        }
+    }
+
+    /**
+     * Tries to register the Guava converters class.
+     */
+    private void tryRegisterGuava() {
+        try {
+            // Guava is not a direct dependency, which is significant in the Java 9 module system
+            // to access Guava this module must add a read edge to the module graph
+            // but since this code is written for Java 6, we have to do this by reflection
+            // yuck
+            Class<?> moduleClass = Class.class.getMethod("getModule").getReturnType();
+            Object convertModule = Class.class.getMethod("getModule").invoke(StringConvert.class);
+            Object layer = convertModule.getClass().getMethod("getLayer").invoke(convertModule);
+            if (layer != null) {
+                Object optGuava = layer.getClass().getMethod("findModule", String.class).invoke(layer, "com.google.common");
+                boolean found = (Boolean) optGuava.getClass().getMethod("isPresent").invoke(optGuava);
+                if (found) {
+                    Object guavaModule = optGuava.getClass().getMethod("get").invoke(optGuava);
+                    moduleClass.getMethod("addReads", moduleClass).invoke(convertModule, guavaModule);
+                }
+            }
+
+        } catch (Throwable ex) {
+            if (LOG) {
+                System.err.println("tryRegisterGuava1: " + ex);
+            }
+        }
+        try {
+            // can now check for Guava
+            // if we have created a read edge, or if we are on the classpath, this will succeed
+            loadType("com.google.common.reflect.TypeToken");
+            @SuppressWarnings("unchecked")
+            Class<?> cls = (Class<TypedStringConverter<?>>) loadType("org.joda.convert.TypeTokenStringConverter");
+            TypedStringConverter<?> conv = (TypedStringConverter<?>) cls.getDeclaredConstructor().newInstance();
+            registered.put(conv.getEffectiveType(), conv);
+
+        } catch (Throwable ex) {
+            if (LOG) {
+                System.err.println("tryRegisterGuava2: " + ex);
+            }
+        }
+    }
+
+    /**
+     * Tries to register the Java 8 optional classes.
+     */
+    private void tryRegisterJava8Optionals() {
+        try {
+            loadType("java.util.OptionalInt");
+            @SuppressWarnings("unchecked")
+            Class<?> cls1 = (Class<TypedStringConverter<?>>) loadType("org.joda.convert.OptionalIntStringConverter");
+            TypedStringConverter<?> conv1 = (TypedStringConverter<?>) cls1.getDeclaredConstructor().newInstance();
+            registered.put(conv1.getEffectiveType(), conv1);
+
+            @SuppressWarnings("unchecked")
+            Class<?> cls2 = (Class<TypedStringConverter<?>>) loadType("org.joda.convert.OptionalLongStringConverter");
+            TypedStringConverter<?> conv2 = (TypedStringConverter<?>) cls2.getDeclaredConstructor().newInstance();
+            registered.put(conv2.getEffectiveType(), conv2);
+
+            @SuppressWarnings("unchecked")
+            Class<?> cls3 = (Class<TypedStringConverter<?>>) loadType("org.joda.convert.OptionalDoubleStringConverter");
+            TypedStringConverter<?> conv3 = (TypedStringConverter<?>) cls3.getDeclaredConstructor().newInstance();
+            registered.put(conv3.getEffectiveType(), conv3);
+
+        } catch (Throwable ex) {
+            if (LOG) {
+                System.err.println("tryRegisterOptionals: " + ex);
+            }
+        }
+    }
+
+    /**
+     * Tries to register the subclasses of TimeZone.
+     * Try various things, doesn't matter if the map entry gets overwritten.
+     */
+    private void tryRegisterTimeZone() {
+        try {
+            registered.put(SimpleTimeZone.class, JDKStringConverter.TIME_ZONE);
+
+        } catch (Throwable ex) {
+            if (LOG) {
+                System.err.println("tryRegisterTimeZone1: " + ex);
+            }
+        }
+        try {
+            TimeZone zone = TimeZone.getDefault();
+            registered.put(zone.getClass(), JDKStringConverter.TIME_ZONE);
+
+        } catch (Throwable ex) {
+            if (LOG) {
+                System.err.println("tryRegisterTimeZone2: " + ex);
+            }
+        }
+        try {
+            TimeZone zone = TimeZone.getTimeZone("Europe/London");
+            registered.put(zone.getClass(), JDKStringConverter.TIME_ZONE);
+
+        } catch (Throwable ex) {
+            if (LOG) {
+                System.err.println("tryRegisterTimeZone3: " + ex);
+            }
+        }
+    }
+
+    /**
+     * Tries to register Java 8 classes.
+     */
+    private void tryRegisterJava8() {
+        try {
             tryRegister("java.time.Instant", "parse");
             tryRegister("java.time.Duration", "parse");
             tryRegister("java.time.LocalDate", "parse");
@@ -175,7 +312,20 @@ public final class StringConvert {
             tryRegister("java.time.Period", "parse");
             tryRegister("java.time.ZoneOffset", "of");
             tryRegister("java.time.ZoneId", "of");
-            // ThreeTen backport classes
+            tryRegister("java.time.ZoneRegion", "of");
+
+        } catch (Throwable ex) {
+            if (LOG) {
+                System.err.println("tryRegisterJava8: " + ex);
+            }
+        }
+    }
+
+    /**
+     * Tries to register ThreeTen backport classes.
+     */
+    private void tryRegisterThreeTenBackport() {
+        try {
             tryRegister("org.threeten.bp.Instant", "parse");
             tryRegister("org.threeten.bp.Duration", "parse");
             tryRegister("org.threeten.bp.LocalDate", "parse");
@@ -190,7 +340,20 @@ public final class StringConvert {
             tryRegister("org.threeten.bp.Period", "parse");
             tryRegister("org.threeten.bp.ZoneOffset", "of");
             tryRegister("org.threeten.bp.ZoneId", "of");
-            // Old ThreeTen/JSR-310 classes v0.6.3 and beyond
+            tryRegister("org.threeten.bp.ZoneRegion", "of");
+
+        } catch (Throwable ex) {
+            if (LOG) {
+                System.err.println("tryRegisterThreeTenBackport: " + ex);
+            }
+        }
+    }
+
+    /**
+     * Tries to register ThreeTen ThreeTen/JSR-310 classes v0.6.3 and beyond.
+     */
+    private void tryRegisterThreeTenOld() {
+        try {
             tryRegister("javax.time.Instant", "parse");
             tryRegister("javax.time.Duration", "parse");
             tryRegister("javax.time.calendar.LocalDate", "parse");
@@ -207,92 +370,11 @@ public final class StringConvert {
             tryRegister("javax.time.calendar.ZoneOffset", "of");
             tryRegister("javax.time.calendar.ZoneId", "of");
             tryRegister("javax.time.calendar.TimeZone", "of");
-        }
-        if (factories.length > 0) {
-            this.factories.addAll(Arrays.asList(factories));
-        }
-        this.factories.add(AnnotationStringConverterFactory.INSTANCE);
-        if (includeJdkConverters) {
-            this.factories.add(EnumStringConverterFactory.INSTANCE);
-        }
-    }
-
-    /**
-     * Tries to register the Guava converters class.
-     */
-    private void tryRegisterGuava() {
-        try {
-            RenameHandler.INSTANCE.loadType("com.google.common.reflect.Types");
-            @SuppressWarnings("unchecked")
-            Class<?> cls = (Class<TypedStringConverter<?>>) RenameHandler.INSTANCE
-                    .loadType("org.joda.convert.TypeTokenStringConverter");
-            TypedStringConverter<?> conv = (TypedStringConverter<?>) cls.newInstance();
-            registered.put(conv.getEffectiveType(), conv);
-
-            @SuppressWarnings("unchecked")
-            Class<?> cls2 = (Class<TypedStringConverter<?>>) RenameHandler.INSTANCE
-                    .loadType("org.joda.convert.TypeStringConverter");
-            TypedStringConverter<?> conv2 = (TypedStringConverter<?>) cls2.newInstance();
-            registered.put(conv2.getEffectiveType(), conv2);
 
         } catch (Throwable ex) {
-            // ignore
-        }
-    }
-
-    /**
-     * Tries to register the Java 8 optional classes.
-     */
-    private void tryRegisterJava8Optionals() {
-        try {
-            RenameHandler.INSTANCE.loadType("java.util.OptionalInt");
-            @SuppressWarnings("unchecked")
-            Class<?> cls1 = (Class<TypedStringConverter<?>>) RenameHandler.INSTANCE
-                    .loadType("org.joda.convert.OptionalIntStringConverter");
-            TypedStringConverter<?> conv1 = (TypedStringConverter<?>) cls1.newInstance();
-            registered.put(conv1.getEffectiveType(), conv1);
-
-            @SuppressWarnings("unchecked")
-            Class<?> cls2 = (Class<TypedStringConverter<?>>) RenameHandler.INSTANCE
-                    .loadType("org.joda.convert.OptionalLongStringConverter");
-            TypedStringConverter<?> conv2 = (TypedStringConverter<?>) cls2.newInstance();
-            registered.put(conv2.getEffectiveType(), conv2);
-
-            @SuppressWarnings("unchecked")
-            Class<?> cls3 = (Class<TypedStringConverter<?>>) RenameHandler.INSTANCE
-                    .loadType("org.joda.convert.OptionalDoubleStringConverter");
-            TypedStringConverter<?> conv3 = (TypedStringConverter<?>) cls3.newInstance();
-            registered.put(conv3.getEffectiveType(), conv3);
-
-        } catch (Throwable ex) {
-            // ignore
-        }
-    }
-
-    /**
-     * Tries to register the subclasses of TimeZone.
-     * Try various things, doesn't matter if the map entry gets overwritten.
-     */
-    private void tryRegisterTimeZone() {
-        try {
-            registered.put(SimpleTimeZone.class, JDKStringConverter.TIME_ZONE);
-
-        } catch (Throwable ex) {
-            // ignore
-        }
-        try {
-            TimeZone zone = TimeZone.getDefault();
-            registered.put(zone.getClass(), JDKStringConverter.TIME_ZONE);
-
-        } catch (Throwable ex) {
-            // ignore
-        }
-        try {
-            TimeZone zone = TimeZone.getTimeZone("Europe/London");
-            registered.put(zone.getClass(), JDKStringConverter.TIME_ZONE);
-
-        } catch (Throwable ex) {
-            // ignore
+            if (LOG) {
+                System.err.println("tryRegisterThreeTenOld: " + ex);
+            }
         }
     }
 
@@ -300,14 +382,11 @@ public final class StringConvert {
      * Tries to register a class using the standard toString/parse pattern.
      * 
      * @param className  the class name, not null
+     * @throws ClassNotFoundException if the class does not exist
      */
-    private void tryRegister(String className, String fromStringMethodName) {
-        try {
-            Class<?> cls = RenameHandler.INSTANCE.lookupType(className);
-            registerMethods(cls, "toString", fromStringMethodName);
-        } catch (Throwable ex) {
-            // ignore
-        }
+    private void tryRegister(String className, String fromStringMethodName) throws ClassNotFoundException {
+        Class<?> cls = loadType(className);
+        registerMethods(cls, "toString", fromStringMethodName);
     }
 
     //-----------------------------------------------------------------------
@@ -770,6 +849,41 @@ public final class StringConvert {
         }
     }
 
+    //-----------------------------------------------------------------------
+    // loads a type avoiding nulls, context class loader if available
+    static Class<?> loadType(String fullName) throws ClassNotFoundException {
+        try {
+            ClassLoader loader = Thread.currentThread().getContextClassLoader();
+            return loader != null && !fullName.startsWith("[") ? loader.loadClass(fullName) : Class.forName(fullName);
+        } catch (ClassNotFoundException ex) {
+            return loadPrimitiveType(fullName, ex);
+        }
+    }
+
+    // handle primitive types
+    private static Class<?> loadPrimitiveType(String fullName, ClassNotFoundException ex) throws ClassNotFoundException {
+        if (fullName.equals("int")) {
+            return int.class;
+        } else if (fullName.equals("long")) {
+            return long.class;
+        } else if (fullName.equals("double")) {
+            return double.class;
+        } else if (fullName.equals("boolean")) {
+            return boolean.class;
+        } else if (fullName.equals("short")) {
+            return short.class;
+        } else if (fullName.equals("byte")) {
+            return byte.class;
+        } else if (fullName.equals("char")) {
+            return char.class;
+        } else if (fullName.equals("float")) {
+            return float.class;
+        } else if (fullName.equals("void")) {
+            return void.class;
+        }
+        throw ex;
+    }
+
     //-----------------------------------------------------------------------
     /**
      * Returns a simple string representation of the object.
